@@ -13,8 +13,9 @@ from cryptography.hazmat.primitives.serialization import (
 
 from .algs.ec2 import EC2Key
 from .algs.okp import OKPKey
+from .algs.raw import RawKey
 from .algs.rsa import RSAKey
-from .algs.symmetric import AESCCMKey, AESGCMKey, ChaCha20Key, HMACKey
+from .algs.symmetric import AESCCMKey, AESGCMKey, AESKeyWrap, ChaCha20Key, HMACKey
 from .const import (
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
     COSE_ALGORITHMS_RSA,
@@ -34,12 +35,12 @@ class COSEKey:
     """
 
     @staticmethod
-    def new(cose_key: Dict[int, Any]) -> COSEKeyInterface:
+    def new(params: Dict[int, Any]) -> COSEKeyInterface:
         """
         Create a COSE key from a CBOR-like dictionary with numeric keys.
 
         Args:
-            cose_key (Dict[int, Any]): A CBOR-like dictionary with numeric keys
+            params (Dict[int, Any]): A CBOR-like dictionary with numeric keys
                 of a COSE key.
         Returns:
             COSEKeyInterface: A COSE key object.
@@ -48,37 +49,39 @@ class COSEKey:
         """
 
         # Validate COSE Key common parameters.
-        if 1 not in cose_key:
+        if 1 not in params:
             raise ValueError("kty(1) not found.")
-        if not isinstance(cose_key[1], int) and not isinstance(cose_key[1], str):
+        if not isinstance(params[1], int) and not isinstance(params[1], str):
             raise ValueError("kty(1) should be int or str(tstr).")
-        if cose_key[1] == 1:
-            return OKPKey(cose_key)
-        if cose_key[1] == 2:
-            return EC2Key(cose_key)
-        if cose_key[1] == 3:
-            return RSAKey(cose_key)
-        if cose_key[1] == 4:
-            if 3 not in cose_key or (
-                not isinstance(cose_key[3], int) and not isinstance(cose_key[3], str)
+        if params[1] == 1:
+            return OKPKey(params)
+        if params[1] == 2:
+            return EC2Key(params)
+        if params[1] == 3:
+            return RSAKey(params)
+        if params[1] == 4:
+            if 3 not in params or (
+                not isinstance(params[3], int) and not isinstance(params[3], str)
             ):
                 raise ValueError("alg(3) should be int or str(tstr).")
-            if cose_key[3] in [1, 2, 3]:
-                return AESGCMKey(cose_key)
-            if cose_key[3] in [4, 5, 6, 7]:
-                return HMACKey(cose_key)
-            if cose_key[3] in [10, 11, 12, 13, 30, 31, 32, 33]:
-                return AESCCMKey(cose_key)
-            if cose_key[3] == 24:
-                return ChaCha20Key(cose_key)
-            raise ValueError(f"Unsupported or unknown alg(3): {cose_key[3]}.")
-        raise ValueError(f"Unsupported or unknown kty(1): {cose_key[1]}.")
+            if params[3] in [1, 2, 3]:
+                return AESGCMKey(params)
+            if params[3] in [4, 5, 6, 7]:
+                return HMACKey(params)
+            if params[3] in [10, 11, 12, 13, 30, 31, 32, 33]:
+                return AESCCMKey(params)
+            if params[3] == 24:
+                return ChaCha20Key(params)
+            if params[3] in [-3, -4, -5]:
+                return AESKeyWrap(params)
+            raise ValueError(f"Unsupported or unknown alg(3): {params[3]}.")
+        raise ValueError(f"Unsupported or unknown kty(1): {params[1]}.")
 
     @classmethod
     def from_symmetric_key(
         cls,
         key: Union[bytes, str] = b"",
-        alg: Union[int, str] = "HMAC 256/256",
+        alg: Union[int, str] = "",
         kid: Union[bytes, str] = b"",
         key_ops: Optional[Union[List[int], List[str]]] = None,
     ) -> COSEKeyInterface:
@@ -101,11 +104,13 @@ class COSEKey:
         """
         if isinstance(key, str):
             key = key.encode("utf-8")
+        if alg == "":
+            return RawKey({1: 4, -1: key})
         alg_id = alg if isinstance(alg, int) else COSE_ALGORITHMS_SYMMETRIC.get(alg, 0)
         if alg_id == 0:
             raise ValueError(f"Unsupported or unknown alg(3): {alg}.")
 
-        cose_key = {
+        params = {
             1: 4,  # kty: 'Symmetric'
             3: alg_id,  # alg: int
             -1: key,  # k: bstr
@@ -113,7 +118,7 @@ class COSEKey:
         if isinstance(kid, str):
             kid = kid.encode("utf-8")
         if kid:
-            cose_key[2] = kid
+            params[2] = kid
 
         key_ops_labels: List[int] = []
         if key_ops and isinstance(key_ops, list):
@@ -125,8 +130,8 @@ class COSEKey:
                         key_ops_labels.append(ops)
             except Exception:
                 raise ValueError("Unsupported or unknown key_ops.")
-        cose_key[4] = key_ops_labels
-        return cls.new(cose_key)
+        params[4] = key_ops_labels
+        return cls.new(params)
 
     @classmethod
     def from_bytes(cls, key_data: bytes) -> COSEKeyInterface:
@@ -141,8 +146,8 @@ class COSEKey:
             ValueError: Invalid arguments.
             DecodeError: Failed to decode the key data.
         """
-        cose_key = cbor2.loads(key_data)
-        return cls.new(cose_key)
+        params = cbor2.loads(key_data)
+        return cls.new(params)
 
     @classmethod
     def from_jwk(cls, data: Union[str, bytes, Dict[str, Any]]) -> COSEKeyInterface:
@@ -199,11 +204,11 @@ class COSEKey:
         else:
             raise ValueError("Failed to decode PEM.")
 
-        cose_key: Dict[int, Any] = {}
+        params: Dict[int, Any] = {}
         if isinstance(kid, str):
             kid = kid.encode("utf-8")
         if kid:
-            cose_key[2] = kid
+            params[2] = kid
 
         key_ops_labels: List[int] = []
         if key_ops and isinstance(key_ops, list):
@@ -215,7 +220,7 @@ class COSEKey:
                         key_ops_labels.append(ops)
             except Exception:
                 raise ValueError("Unsupported or unknown key_ops.")
-        cose_key[4] = key_ops_labels
+        params[4] = key_ops_labels
 
         if isinstance(k, RSAPublicKey) or isinstance(k, RSAPrivateKey):
             if not alg:
@@ -224,22 +229,22 @@ class COSEKey:
                 if alg not in COSE_ALGORITHMS_RSA:
                     raise ValueError(f"Unsupported or unknown alg: {alg}.")
                 alg = COSE_ALGORITHMS_RSA[alg]
-            cose_key[1] = COSE_KEY_TYPES["RSA"]
-            cose_key[3] = alg
+            params[1] = COSE_KEY_TYPES["RSA"]
+            params[3] = alg
             if isinstance(k, RSAPublicKey):
                 pub_nums = k.public_numbers()
-                cose_key[-1] = uint_to_bytes(pub_nums.n)
-                cose_key[-2] = uint_to_bytes(pub_nums.e)
+                params[-1] = uint_to_bytes(pub_nums.n)
+                params[-2] = uint_to_bytes(pub_nums.e)
             else:
                 priv_nums = k.private_numbers()
-                cose_key[-1] = uint_to_bytes(priv_nums.public_numbers.n)
-                cose_key[-2] = uint_to_bytes(priv_nums.public_numbers.e)
-                cose_key[-3] = uint_to_bytes(priv_nums.d)
-                cose_key[-4] = uint_to_bytes(priv_nums.p)
-                cose_key[-5] = uint_to_bytes(priv_nums.q)
-                cose_key[-6] = uint_to_bytes(priv_nums.dmp1)  # dP
-                cose_key[-7] = uint_to_bytes(priv_nums.dmq1)  # dQ
-                cose_key[-8] = uint_to_bytes(priv_nums.iqmp)  # qInv
+                params[-1] = uint_to_bytes(priv_nums.public_numbers.n)
+                params[-2] = uint_to_bytes(priv_nums.public_numbers.e)
+                params[-3] = uint_to_bytes(priv_nums.d)
+                params[-4] = uint_to_bytes(priv_nums.p)
+                params[-5] = uint_to_bytes(priv_nums.q)
+                params[-6] = uint_to_bytes(priv_nums.dmp1)  # dP
+                params[-7] = uint_to_bytes(priv_nums.dmq1)  # dQ
+                params[-8] = uint_to_bytes(priv_nums.iqmp)  # qInv
 
         elif isinstance(k, EllipticCurvePrivateKey) or isinstance(
             k, EllipticCurvePublicKey
@@ -252,8 +257,8 @@ class COSEKey:
                         alg = COSE_ALGORITHMS_SIG_EC2[alg]
                     else:
                         raise ValueError(f"Unsupported or unknown alg for EC2: {alg}.")
-                cose_key[3] = alg
-            cose_key.update(EC2Key.to_cose_key(k))
+                params[3] = alg
+            params.update(EC2Key.to_cose_key(k))
         else:
             if alg:
                 if isinstance(alg, str):
@@ -263,6 +268,6 @@ class COSEKey:
                         alg = COSE_ALGORITHMS_SIG_OKP[alg]
                     else:
                         raise ValueError(f"Unsupported or unknown alg for OKP: {alg}.")
-                cose_key[3] = alg
-            cose_key.update(OKPKey.to_cose_key(k))
-        return cls.new(cose_key)
+                params[3] = alg
+            params.update(OKPKey.to_cose_key(k))
+        return cls.new(params)
