@@ -1,20 +1,24 @@
 import json
 from typing import Any, Dict, List, Optional, Union
 
+import cbor2
+
 from .const import (  # COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP,
+    COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT,
+    COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP,
     COSE_ALGORITHMS_KEY_WRAP,
     COSE_ALGORITHMS_RECIPIENT,
-    COSE_KEY_OPERATION_VALUES,
 )
 from .cose_key import COSEKey
 from .cose_key_interface import COSEKeyInterface
 from .recipient_algs.aes_key_wrap import AESKeyWrap
 from .recipient_algs.direct_hkdf import DirectHKDF
 from .recipient_algs.direct_key import DirectKey
+from .recipient_algs.ecdh_aes_key_wrap import ECDH_AESKeyWrap
 from .recipient_algs.ecdh_direct_hkdf import ECDH_DirectHKDF
 from .recipient_interface import RecipientInterface
-from .utils import base64url_decode, to_cose_header
+from .utils import to_cose_header
 
 
 class Recipient:
@@ -29,9 +33,7 @@ class Recipient:
         unprotected: dict = {},
         ciphertext: bytes = b"",
         recipients: List[Any] = [],
-        key_ops: List[int] = [],
-        key: bytes = b"",
-        cose_key: Optional[COSEKeyInterface] = None,
+        sender_key: Optional[COSEKeyInterface] = None,
     ) -> RecipientInterface:
         """
         Create a recipient from a CBOR-like dictionary with numeric keys.
@@ -55,11 +57,13 @@ class Recipient:
         if alg in [-10, -11]:
             return DirectHKDF(p, u, ciphertext, recipients)
         if alg in [-3, -4, -5]:
-            return AESKeyWrap(p, u, ciphertext, recipients, key_ops, key)
+            if not sender_key:
+                sender_key = COSEKey.from_symmetric_key(alg=alg)
+            return AESKeyWrap(p, u, sender_key, ciphertext, recipients)
         if alg in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT.values():
-            return ECDH_DirectHKDF(p, u, ciphertext, recipients, cose_key)
-        # if alg in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP:
-        #     return ECDH_AESKeyWrap(p, u, ciphertext, recipients, key_ops, key)
+            return ECDH_DirectHKDF(p, u, ciphertext, recipients, sender_key)
+        if alg in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP.values():
+            return ECDH_AESKeyWrap(p, u, ciphertext, recipients, sender_key)
         raise ValueError(f"Unsupported or unknown alg(1): {alg}.")
 
     @classmethod
@@ -78,8 +82,6 @@ class Recipient:
         protected: Dict[int, Any] = {}
         unprotected: Dict[int, Any] = {}
         recipient: Dict[str, Any]
-        key = b""
-        key_ops = []
 
         if not isinstance(data, dict):
             recipient = json.loads(data)
@@ -93,7 +95,7 @@ class Recipient:
             unprotected[-20] = recipient["salt"].encode("utf-8")
 
         # alg
-        cose_key = None
+        sender_key = None
         if "alg" in recipient:
             if not isinstance(recipient["alg"], str):
                 raise ValueError("alg should be str.")
@@ -103,10 +105,11 @@ class Recipient:
                 unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
             elif recipient["alg"] in COSE_ALGORITHMS_KEY_WRAP:
                 unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
+                sender_key = COSEKey.from_jwk(recipient)
             else:
                 protected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
-            if recipient["alg"] in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT.keys():
-                cose_key = COSEKey.from_jwk(recipient)
+            if recipient["alg"] in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT.keys():
+                sender_key = COSEKey.from_jwk(recipient)
 
         # kid
         if "kid" in recipient:
@@ -114,24 +117,26 @@ class Recipient:
                 raise ValueError("kid should be str.")
             unprotected[4] = recipient["kid"].encode("utf-8")
 
-        # key_ops
-        if "key_ops" in recipient:
-            if not isinstance(recipient["key_ops"], list):
-                raise ValueError("key_ops should be list.")
-            for ops in recipient["key_ops"]:
-                if not isinstance(ops, str):
-                    raise ValueError("Each value of key_ops should be str.")
-                try:
-                    key_ops.append(COSE_KEY_OPERATION_VALUES[ops])
-                except Exception:
-                    raise ValueError(f"Unknown key_ops: {ops}.")
+        return cls.new(protected, unprotected, sender_key=sender_key)
 
-        # k
-        if "k" in recipient:
-            if not isinstance(recipient["k"], str):
-                raise ValueError("k should be str.")
-            key = base64url_decode(recipient["k"])
-
-        return cls.new(
-            protected, unprotected, key_ops=key_ops, key=key, cose_key=cose_key
-        )
+    @classmethod
+    def from_list(cls, recipient: List[Any]) -> RecipientInterface:
+        if not isinstance(recipient, list) or (
+            len(recipient) != 3 and len(recipient) != 4
+        ):
+            raise ValueError("Invalid recipient format.")
+        if not isinstance(recipient[0], bytes):
+            raise ValueError("protected header should be bytes.")
+        protected = {} if not recipient[0] else cbor2.loads(recipient[0])
+        if not isinstance(recipient[1], dict):
+            raise ValueError("unprotected header should be dict.")
+        if not isinstance(recipient[2], bytes):
+            raise ValueError("ciphertext should be bytes.")
+        if len(recipient) == 3:
+            return Recipient.new(protected, recipient[1], recipient[2])
+        if not isinstance(recipient[3], list):
+            raise ValueError("recipients should be list.")
+        recipients: List[RecipientInterface] = []
+        for r in recipient[3]:
+            recipients.append(cls.from_list(r))
+        return cls.new(protected, recipient[1], recipient[2], recipients)
