@@ -1,9 +1,16 @@
 from typing import Any, Dict, List, Optional, Union
 
+from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePublicKey
+from cryptography.hazmat.primitives.asymmetric.x448 import X448PublicKey
+from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PublicKey
+
+from .algs.ec2 import EC2Key
+from .algs.okp import OKPKey
+from .cbor_processor import CBORProcessor
 from .cose_key_interface import COSEKeyInterface
 
 
-class RecipientInterface(COSEKeyInterface):
+class RecipientInterface(CBORProcessor):
     """
     The interface class for a COSE Recipient.
     """
@@ -33,45 +40,35 @@ class RecipientInterface(COSEKeyInterface):
         """
         protected = {} if protected is None else protected
         unprotected = {} if unprotected is None else unprotected
-
-        params: Dict[int, Any] = {1: 4}  # Support only Symmetric key.
+        self._alg = 0
 
         # kid
         if 4 in unprotected:
             if not isinstance(unprotected[4], bytes):
                 raise ValueError("unprotected[4](kid) should be bytes.")
-            params[2] = unprotected[4]
+        self._kid = unprotected[4] if 4 in unprotected else None
 
         # alg
         if 1 in protected:
             if not isinstance(protected[1], int):
                 raise ValueError("protected[1](alg) should be int.")
-            params[3] = protected[1]
+            self._alg = protected[1]
         elif 1 in unprotected:
             if not isinstance(unprotected[1], int):
                 raise ValueError("unprotected[1](alg) should be int.")
-            params[3] = unprotected[1]
-            if params[3] == -6:  # direct
+            self._alg = unprotected[1]
+            if unprotected[1] == -6:  # direct
                 if len(protected) != 0:
                     raise ValueError("protected header should be empty.")
                 if len(ciphertext) != 0:
                     raise ValueError("ciphertext should be zero-length bytes.")
                 if len(recipients) != 0:
                     raise ValueError("recipients should be absent.")
-        else:
-            params[3] = 0
-
-        # key_ops
-        if key_ops:
-            params[4] = key_ops
 
         # iv
         if 5 in unprotected:
             if not isinstance(unprotected[5], bytes):
                 raise ValueError("unprotected[5](iv) should be bytes.")
-            params[5] = unprotected[5]
-
-        super().__init__(params)
 
         self._protected = protected
         self._unprotected = unprotected
@@ -89,11 +86,12 @@ class RecipientInterface(COSEKeyInterface):
         return
 
     @property
-    def key(self) -> bytes:
-        """
-        The body of the key as bytes.
-        """
-        return self._key
+    def kid(self) -> bytes:
+        return self._kid
+
+    @property
+    def alg(self) -> int:
+        return self._alg
 
     @property
     def protected(self) -> Dict[int, Any]:
@@ -142,60 +140,70 @@ class RecipientInterface(COSEKeyInterface):
         res.append(children)
         return res
 
-    def set_key(self, key: bytes):
-        """
-        Sets a key.
-
-        Args:
-            key (bytes): The key as bytes.
-        """
-        self._key = key
-        return
-
-    def verify_key(
+    def apply(
         self,
-        material: bytes,
-        expected_key: bytes,
-        context: Union[List[Any], Dict[str, Any]],
-    ):
+        key: Optional[COSEKeyInterface] = None,
+        recipient_key: Optional[COSEKeyInterface] = None,
+        alg: Optional[int] = None,
+        context: Optional[Union[List[Any], Dict[str, Any]]] = None,
+    ) -> COSEKeyInterface:
         """
-        Verifies a key with a key material and an expected key.
+        Applies a COSEKey as a material to prepare a MAC/encryption key with
+        the recipient-specific method (e.g., key wrapping, key agreement,
+        or the combination of them) and sets up the related information
+        (context information or ciphertext) in the recipient structure.
+        Therefore, it will be used by the sender of the recipient information
+        before calling COSE.encode_* functions with the Recipient object. The
+        key generated through this function will be set to ``key`` parameter
+        of COSE.encode_* functions.
 
         Args:
-            material (bytes): A key material.
-            expected_key (bytes): A byte string of the expected key.
-            context (Union[List[Any], Dict[str, Any]]): Context information structure.
-        Raises:
-            NotImplementedError: Not implemented.
-            ValueError: Invalid arguments.
-            VerifyError: Failed to verify the key.
-        """
-        raise NotImplementedError
-
-    def wrap_key(self, key_to_wrap: bytes):
-        """
-        Wraps a key and keeps it internally as the ciphertext.
-
-        Args:
-            key_to_wrap (bytes): A key to be wrapped.
-        Raises:
-            NotImplementedError: Not implemented.
-            ValueError: Invalid arguments.
-            EncodeError: Failed to encode(wrap) key.
-        """
-        raise NotImplementedError
-
-    def unwrap_key(self, alg: int) -> COSEKeyInterface:
-        """
-        Unwraps the key stored as the ciphertext.
-
-        Args:
-            alg (int): The algorithm of the wrapped key.
+            key (Optional[COSEKeyInterface]): The external key to
+                be used for preparing the key.
+            recipient_key (Optional[COSEKeyInterface]): The external public
+                key provided by the recipient used for ECDH key agreement.
+            alg (Optional[int]): The algorithm of the key to be generated.
+            context (Optional[Union[List[Any], Dict[str, Any]]]): Context
+                information structure.
         Returns:
-            COSEKeyInterface: An unwrapped key.
+            COSEKeyInterface: A generated key or passed-throug key which is used
+                as ``key`` parameter of COSE.encode_* functions.
         Raises:
-            NotImplementedError: Not implemented.
             ValueError: Invalid arguments.
-            DecodeError: Failed to decode(unwrap) the key.
+            EncodeError: Failed to encode(e.g., wrap, derive) the key.
         """
         raise NotImplementedError
+
+    def extract(
+        self,
+        key: COSEKeyInterface,
+        alg: Optional[int] = None,
+        context: Optional[Union[List[Any], Dict[str, Any]]] = None,
+    ) -> COSEKeyInterface:
+        """
+        Extracts a MAC/encryption key with the recipient-specific method
+        (e.g., key wrapping, key agreement, or the combination of them).
+        This function will be called in COSE.decode so applications do
+        not need to call it directly.
+
+        Args:
+            key (COSEKeyInterface): The external key to be used for
+                extracting the key.
+            alg (Optional[int]): The algorithm of the key extracted.
+            context (Optional[Union[List[Any], Dict[str, Any]]]): Context
+                information structure.
+        Returns:
+            COSEKeyInterface: An extracted key which is used for decrypting
+                or verifying a payload message.
+        Raises:
+            ValueError: Invalid arguments.
+            DecodeError: Failed to decode(e.g., unwrap, derive) the key.
+        """
+        raise NotImplementedError
+
+    def _to_cose_key(
+        self, k: Union[EllipticCurvePublicKey, X25519PublicKey, X448PublicKey]
+    ) -> Dict[int, Any]:
+        if isinstance(k, EllipticCurvePublicKey):
+            return EC2Key.to_cose_key(k)
+        return OKPKey.to_cose_key(k)
