@@ -545,16 +545,18 @@ class TestCOSE:
                 "salt": "aabbccddeeffgghh",
             },
         )
+        material = COSEKey.from_symmetric_key(
+            key=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
+            alg="A256GCM",
+            kid="our-secret",
+        )
         context = [
             10,
             [b"lighting-client", None, None],
             [b"lighting-server", None, None],
             [128, cbor2.dumps({1: -10}), b"Encryption Example 02"],
         ]
-        enc_key = recipient.derive_key(
-            context,
-            material=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
-        )
+        enc_key = recipient.apply(material, context=context)
         ctx = COSE.new()
         encoded = ctx.encode_and_encrypt(
             b"This is the content.",
@@ -564,10 +566,6 @@ class TestCOSE:
             recipients=[recipient],
         )
         assert encoded == bytes.fromhex(cwt_str)
-        material = {
-            "kid": "our-secret",
-            "value": "hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg",
-        }
         context = {
             "alg": "AES-CCM-16-64-128",
             "apu": {
@@ -580,7 +578,7 @@ class TestCOSE:
                 "other": "Encryption Example 02",
             },
         }
-        res = ctx.decode(encoded, context=context, materials=[material])
+        res = ctx.decode(encoded, context=context, keys=[material])
         assert res == b"This is the content."
 
     def test_cose_sample_cose_wg_rfc8152_c_3_2_with_json(self):
@@ -604,10 +602,11 @@ class TestCOSE:
                 "other": "Encryption Example 02",
             },
         }
-        enc_key = recipient.derive_key(
-            context,
-            material=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
+        material = COSEKey.from_symmetric_key(
+            base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
+            alg="A256GCM",
         )
+        enc_key = recipient.apply(material, context=context)
         ctx = COSE.new()
         encoded = ctx.encode_and_encrypt(
             b"This is the content.",
@@ -617,10 +616,11 @@ class TestCOSE:
             recipients=[recipient],
         )
         assert encoded == bytes.fromhex(cwt_str)
-        material = {
-            "kid": "our-secret",
-            "value": "hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg",
-        }
+        material = COSEKey.from_symmetric_key(
+            key=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
+            alg="A256GCM",
+            kid="our-secret",
+        )
         context = {
             "alg": "AES-CCM-16-64-128",
             "apu": {
@@ -633,7 +633,7 @@ class TestCOSE:
                 "other": "Encryption Example 02",
             },
         }
-        res = ctx.decode(encoded, context=context, materials=[material])
+        res = ctx.decode(encoded, context=context, keys=[material])
         assert res == b"This is the content."
 
     def test_cose_sample_cose_wg_aes_wrap_128_03(self):
@@ -646,12 +646,13 @@ class TestCOSE:
         )
         recipient = Recipient.from_jwk(
             {
+                "kty": "oct",
                 "alg": "A128KW",
                 "kid": "our-secret",
                 "k": "hJtXIZ2uSN5kbQfbtTNWbg",
             },
         )
-        recipient.wrap_key(mac_key.key)
+        recipient.apply(mac_key)
         ctx = COSE.new()
         encoded = ctx.encode_and_mac(
             b"This is the content.",
@@ -660,15 +661,23 @@ class TestCOSE:
             recipients=[recipient],
         )
         assert encoded == bytes.fromhex(cwt_str)
-        res = ctx.decode(encoded, key=[recipient])
+        key = COSEKey.from_jwk(
+            {
+                "kty": "oct",
+                "alg": "A128KW",
+                "kid": "our-secret",
+                "k": "hJtXIZ2uSN5kbQfbtTNWbg",
+            },
+        )
+        res = ctx.decode(encoded, keys=[key])
         assert res == b"This is the content."
 
     def test_cose_sample_cose_wg_ecdh_direct_p256_hkdf_256_01(self):
         rec = Recipient.from_jwk(
             {
                 "kty": "EC",
-                "alg": "ECDH-ES+HKDF-256",
                 "crv": "P-256",
+                "alg": "ECDH-ES+HKDF-256",
             }
         )
         pub_key = COSEKey.from_jwk(
@@ -680,7 +689,7 @@ class TestCOSE:
                 "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
             }
         )
-        enc_key = rec.derive_key({"alg": "A128GCM"}, public_key=pub_key)
+        enc_key = rec.apply(recipient_key=pub_key, context={"alg": "A128GCM"})
         ctx = COSE.new(alg_auto_inclusion=True)
         encoded = ctx.encode_and_encrypt(
             b"This is the content.",
@@ -702,6 +711,57 @@ class TestCOSE:
             encoded, priv_key, context={"alg": "A128GCM"}
         )
 
+    def test_cose_sample_cose_wg_ecdh_wrap_p256_ss_wrap_128_01(self):
+        # The sender side:
+        enc_key = COSEKey.from_symmetric_key(
+            # bytes.fromhex("B2353161740AACF1F7163647984B522A"),
+            alg="A128GCM",
+        )
+        rec = Recipient.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-256",
+                "alg": "ECDH-SS+A128KW",
+                "x": "7cvYCcdU22WCwW1tZXR8iuzJLWGcd46xfxO1XJs-SPU",
+                "y": "DzhJXgz9RI6TseNmwEfLoNVns8UmvONsPzQDop2dKoo",
+                "d": "Uqr4fay_qYQykwcNCB2efj_NFaQRRQ-6fHZm763jt5w",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-256",
+                "kid": "meriadoc.brandybuck@buckland.example",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+                # "d":"r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8"
+            }
+        )
+        rec.apply(enc_key, recipient_key=pub_key, context={"alg": "A128GCM"})
+        ctx = COSE.new(alg_auto_inclusion=True)
+        encoded = ctx.encode_and_encrypt(
+            b"This is the content.",
+            key=enc_key,
+            nonce=b"\x02\xd1\xf7\xe6\xf2lC\xd4\x86\x8d\x87\xce",
+            recipients=[rec],
+        )
+
+        # The recipient side:
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "crv": "P-256",
+                "alg": "ECDH-SS+A128KW",
+                "kid": "meriadoc.brandybuck@buckland.example",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+                "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
+            }
+        )
+        assert b"This is the content." == ctx.decode(
+            encoded, priv_key, context={"alg": "A128GCM"}
+        )
+
     @pytest.mark.parametrize(
         "invalid, msg",
         [
@@ -926,9 +986,9 @@ class TestCOSE:
         key = COSEKey.from_symmetric_key(alg="HS256")
         encoded = cwt.encode({"iss": "coap://as.example"}, key)
         with pytest.raises(ValueError) as err:
-            ctx.decode(encoded)
+            ctx.decode(encoded, b"")
             pytest.fail("decode should fail.")
-        assert "Either key or materials should be specified." in str(err.value)
+        assert "key in keys should have COSEKeyInterface." in str(err.value)
 
     def test_cose_decode_mac0_with_invalid_multiple_keys(self, ctx):
         key1 = COSEKey.from_symmetric_key(alg="HS256")
@@ -980,10 +1040,29 @@ class TestCOSE:
         recipient = Recipient.from_jwk(
             {"kty": "EC", "crv": "P-256", "alg": "ECDH-ES+HKDF-256"}
         )
-        enc_key = recipient.derive_key(
-            {"alg": "A128GCM"},
-            public_key=public_key,
+        enc_key = recipient.apply(recipient_key=public_key, context={"alg": "A128GCM"})
+        ctx = COSE.new(alg_auto_inclusion=True)
+        encoded = ctx.encode_and_encrypt(
+            b"This is the content.",
+            key=enc_key,
+            recipients=[recipient],
         )
+
+        with open(key_path("private_key_es256.pem")) as key_file:
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
+        with pytest.raises(ValueError) as err:
+            ctx.decode(encoded, private_key)
+            pytest.fail("decode should fail.")
+        assert "context should be set." in str(err.value)
+
+    def test_cose_decode_ecdh_aes_key_wrap_without_context(self):
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
+        enc_key = COSEKey.from_symmetric_key(alg="A128GCM")
+        recipient = Recipient.from_jwk(
+            {"kty": "EC", "crv": "P-256", "alg": "ECDH-ES+A128KW"}
+        )
+        recipient.apply(enc_key, recipient_key=public_key, context={"alg": "A128GCM"})
         ctx = COSE.new(alg_auto_inclusion=True)
         encoded = ctx.encode_and_encrypt(
             b"This is the content.",
